class Session:
    json_key = "session"

    def __init__(self):
        self._new = bool()
        self._sessionId = str()
        self._application = dict()
        self._user = dict()
        self._attributes = dict()

    @property
    def new(self):
        return self._new

    @new.setter
    def new(self, new: bool) -> None:
        if not isinstance(new, bool):
            raise Exception(f"new was type {type(new)} which is not valid value for his parameter.")
        self._new = new

    @property
    def sessionId(self) -> str:
        return self._sessionId

    @sessionId.setter
    def sessionId(self, sessionId: str) -> None:
        if not isinstance(sessionId, str):
            raise Exception(f"sessionId was type {type(sessionId)} which is not valid value for his parameter.")
        self._sessionId = sessionId

    @property
    def application(self) -> dict:
        return self._application

    @application.setter
    def application(self, application: dict) -> None:
        if not isinstance(application, dict):
            raise Exception(f"application was type {type(application)} which is not valid value for his parameter.")
        self._application = application

    @property
    def user(self) -> dict:
        return self._user

    @user.setter
    def user(self, user: dict) -> None:
        if not isinstance(user, dict):
            raise Exception(f"user was type {type(user)} which is not valid value for his parameter.")
        self._user = user

    @property
    def attributes(self) -> dict:
        return self._attributes

    @attributes.setter
    def attributes(self, attributes: dict) -> None:
        if not isinstance(attributes, dict):
            raise Exception(f"attributes was type {type(attributes)} which is not valid value for his parameter.")
        self._attributes = attributes



import json

def dynamodb_to_python(dynamodb_item_value_or_dict_or_list):
    from decimal import Decimal
    if isinstance(dynamodb_item_value_or_dict_or_list, Decimal):
        if dynamodb_item_value_or_dict_or_list % 1 > 0:
            return float(dynamodb_item_value_or_dict_or_list)
        else:
            return int(dynamodb_item_value_or_dict_or_list)

    elif isinstance(dynamodb_item_value_or_dict_or_list, list):
        for i, item in enumerate(dynamodb_item_value_or_dict_or_list):
            dynamodb_item_value_or_dict_or_list[i] = dynamodb_to_python(item)
        return dynamodb_item_value_or_dict_or_list

    elif isinstance(dynamodb_item_value_or_dict_or_list, dict):
        for item_key, item_value in dynamodb_item_value_or_dict_or_list.items():
            dynamodb_item_value_or_dict_or_list[item_key] = dynamodb_to_python(item_value)
        return dynamodb_item_value_or_dict_or_list

    else:
        return dynamodb_item_value_or_dict_or_list

def dict_to_dynamodb(item_object: dict):
    from decimal import Decimal
    if isinstance(item_object, int) or isinstance(item_object, float):
        return Decimal(item_object)

    elif isinstance(item_object, str) and item_object.replace("'", "").replace("\"", "") == "":
        return None

    elif isinstance(item_object, list):
        items_to_remove = list()
        for i, item in enumerate(item_object):
            nested_item = dict_to_dynamodb(item)
            if nested_item is not None:
                item_object[i] = nested_item
            else:
                items_to_remove.append(nested_item)

        for item_to_remove in items_to_remove:
            item_object.remove(item_to_remove)
        return item_object

    elif isinstance(item_object, dict):
        keys_to_pop = list()
        for item_key, item_value in item_object.items():
            nested_item = dict_to_dynamodb(item_value)
            if nested_item is not None:
                item_object[item_key] = nested_item
            else:
                keys_to_pop.append(item_key)

        for key_to_pop in keys_to_pop:
            item_object.pop(key_to_pop)
        return item_object

    else:
        return item_object



# from inoft_vocal_framework.platforms_handlers.handler_input import HandlerInput

KEY_INTERACTIONS_HISTORY = "interactions_history"
KEY_PLAYED_CATEGORIES_TYPES_HISTORY = "played_categories_types_history"
TYPE_NAME_NO_CATEGORY_TYPE_PLAYED_IN_CURRENT_SESSION = "NO_CATEGORY_TYPE_PLAYED_IN_CURRENT_SESSION"
HandlerInput = None

def get_session_attr(handler_input: HandlerInput):
    session_attr = handler_input.attributes_manager.session_attributes
    if not isinstance(session_attr, dict):
        handler_input.attributes_manager.session_attributes = dict()
        session_attr = handler_input.attributes_manager.session_attributes
    return session_attr

def set_session_attribute_element(handler_input: HandlerInput, element_key, element_values):
    get_session_attr(handler_input)[element_key] = element_values

def add_new_interaction(handler_input: HandlerInput, new_interaction_name: str):
    session_attr = get_session_attr(handler_input=handler_input)
    if KEY_INTERACTIONS_HISTORY not in session_attr.keys():
        session_attr[KEY_INTERACTIONS_HISTORY] = list()

    if len(session_attr[KEY_INTERACTIONS_HISTORY]) > 1:
        session_attr[KEY_INTERACTIONS_HISTORY].reverse()
    session_attr[KEY_INTERACTIONS_HISTORY].append(new_interaction_name)
    if len(session_attr[KEY_INTERACTIONS_HISTORY]) > 1:
        session_attr[KEY_INTERACTIONS_HISTORY].reverse()

def get_interactions_history(handler_input: HandlerInput):
    return get_session_attr(handler_input)[KEY_INTERACTIONS_HISTORY]

def add_new_played_category(handler_input: HandlerInput, new_played_interactions_types):
    session_attr = get_session_attr(handler_input=handler_input)
    if KEY_PLAYED_CATEGORIES_TYPES_HISTORY not in session_attr.keys():
        session_attr[KEY_PLAYED_CATEGORIES_TYPES_HISTORY] = list()

    if len(session_attr[KEY_PLAYED_CATEGORIES_TYPES_HISTORY]) > 1:
        session_attr[KEY_PLAYED_CATEGORIES_TYPES_HISTORY].reverse()
    session_attr[KEY_PLAYED_CATEGORIES_TYPES_HISTORY].append(new_played_interactions_types)
    if len(session_attr[KEY_PLAYED_CATEGORIES_TYPES_HISTORY]) > 1:
        session_attr[KEY_PLAYED_CATEGORIES_TYPES_HISTORY].reverse()

def get_played_categories_types_history(handler_input: HandlerInput):
    session_attr = get_session_attr(handler_input)
    if KEY_PLAYED_CATEGORIES_TYPES_HISTORY not in session_attr.keys():
        return list(TYPE_NAME_NO_CATEGORY_TYPE_PLAYED_IN_CURRENT_SESSION)
    return get_session_attr(handler_input)[KEY_PLAYED_CATEGORIES_TYPES_HISTORY]

class PlayedCategoriesTypesHistory:
    def __init__(self, handler_input: HandlerInput):
        session_attr = get_session_attr(handler_input)
        if KEY_PLAYED_CATEGORIES_TYPES_HISTORY not in session_attr.keys():
            session_attr[KEY_PLAYED_CATEGORIES_TYPES_HISTORY] = list()
        self.categories_types_history_values = session_attr[KEY_PLAYED_CATEGORIES_TYPES_HISTORY]
        print(f"Played categories types history values = {self.categories_types_history_values}")

    def get_last(self):
        if (self.categories_types_history_values is None
        or not len(self.categories_types_history_values) > 0
        or self.categories_types_history_values[0] is None):
            return list()
        else:
            return self.categories_types_history_values[0]

def get_skill_application_id(handler_input: HandlerInput) -> str:
    return handler_input.request_envelope.context.system.application.application_id



def raise_invalid_variable_type(variable_value, expected_type, variable_name: str = None):
    raise Exception(f"Invalid type. Received {variable_value} of type {type(variable_value)}" +
                    f"for the {variable_name} variable," if variable_name is None else "" +
                    f" but was expecting a variable of type {expected_type}.")

def raise_if_variable_not_expected_type(value, expected_type, variable_name: str = None):
    if not isinstance(value, expected_type):
        raise_invalid_variable_type(variable_value=value, expected_type=expected_type, variable_name=variable_name)

def raise_if_variable_not_expected_type_and_not_none(value, expected_type, variable_name: str = None):
    if value is not None:
        raise_if_variable_not_expected_type(value=value, expected_type=expected_type, variable_name=variable_name)

def raise_if_value_not_in_list(value, list_object: list, variable_name: str = None):
    if not isinstance(list_object, list):
        raise Exception(f"The list object {list_object} must be of type list but was {type(list_object)}")
    if value not in list_object:
        raise Exception(f"The {'variable with value of ' if variable_name is not None else 'value'} {value} "
                        f"was not present in the following list {list_object}")




BUCKET_NAMING_MSG = """
* Bucket names must be unique across all existing bucket names in Amazon S3.
* Bucket names must comply with DNS naming conventions.
* Bucket names must be at least 3 and no more than 63 characters long.
* Bucket names must not contain uppercase characters or underscores.
* Bucket names must start with a lowercase letter or number.
* Bucket names must be a series of one or more labels. Adjacent labels are separated
by a single period (.). Bucket names can contain lowercase letters, numbers, and
hyphens. Each label must start and end with a lowercase letter or a number.
* Bucket names must not be formatted as an IP address (for example, 192.168.5.4).
* When you use virtual hosted–style buckets with Secure Sockets Layer (SSL), the SSL
wildcard certificate only matches buckets that don't contain periods. To work around
this, use HTTP or write your own certificate verification logic. We recommend that
you do not use periods (".") in bucket names when using virtual hosted–style buckets.
"""

# https://github.com/Miserlou/Zappa/issues/1688
def is_valid_bucket_name(name):
    """
    Checks if an S3 bucket name is valid according to https://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html#bucketnamingrules
    """
    # Bucket names must be at least 3 and no more than 63 characters long.
    if len(name) < 3 or len(name) > 63:
        return False
    # Bucket names must not contain uppercase characters or underscores.
    if any(x.isupper() for x in name):
        return False
    if "_" in name:
        return False
    # Bucket names must start with a lowercase letter or number.
    if not (name[0].islower() or name[0].isdigit()):
        return False
    # Bucket names must be a series of one or more labels. Adjacent labels are separated by a single period (.).
    for label in name.split("."):
        # Each label must start and end with a lowercase letter or a number.
        if len(label) < 1:
            return False
        if not (label[0].islower() or label[0].isdigit()):
            return False
        if not (label[-1].islower() or label[-1].isdigit()):
            return False
    # Bucket names must not be formatted as an IP address (for example, 192.168.5.4).
    looks_like_IP = True
    for label in name.split("."):
        if not label.isdigit():
            looks_like_IP = False
            break
    if looks_like_IP:
        return False

    return True




import boto3
from boto3.session import Session, ResourceNotExistsError
from inoft_vocal_framework.utils.static_logger import logger

class DynamoDbCoreAdapter:
    def __init__(self, table_name: str, region_name: str, primary_key_name="id", create_table=True):
        print(f"Initializing the {self}. For local development, make sure that you are connected to internet."
              f"\nOtherwise the framework will get stuck at initializing the {self}")

        self.table_name = table_name
        self.primary_key_name = primary_key_name
        self.create_table = create_table

        dynamodb_regions = Session().get_available_regions("dynamodb")
        if region_name in dynamodb_regions:
            self.dynamodb = boto3.resource("dynamodb", region_name=region_name)
        else:
            self.dynamodb = boto3.resource("dynamodb")
            logger.debug(f"Warning ! The specified dynamodb region_name {region_name} is not a valid region_name."
                         f"The dynamodb client has been initialized without specifying the region.")

        self.__create_table_if_not_exists()
        print(f"Initialization of {self} completed successfully !")

    def _get_db_table(self):
        # todo: find what type of object is a dynamodb Table (if any)
        return self.dynamodb.Table(self.table_name)

    def __create_table_if_not_exists(self) -> None:
        """
        Creates table in Dynamodb resource if it doesn't exist and create_table is set as True.
        :raises: PersistenceException: When `create_table` fails on dynamodb resource.
        """
        if self.create_table:
            try:
                self.dynamodb.create_table(
                    TableName=self.table_name,
                    KeySchema=[
                        {
                            'AttributeName': self.primary_key_name,
                            'KeyType': 'HASH'
                        }
                    ],
                    AttributeDefinitions=[
                        {
                            'AttributeName': self.primary_key_name,
                            'AttributeType': 'S'
                        }
                    ],
                    BillingMode='PAY_PER_REQUEST',
                )
            except Exception as e:
                if e.__class__.__name__ != "ResourceInUseException":
                    raise Exception(f"Create table if not exists request failed: Exception of type {type(e).__name__} occurred {str(e)}")



from inoft_vocal_framework.platforms_handlers.nested_object_to_dict import NestedObjectToDict
from inoft_vocal_framework.platforms_handlers.samsungbixby_v1.request import Request
from inoft_vocal_framework.platforms_handlers.samsungbixby_v1.response import Response


class BixbyHandlerInput:
    from inoft_vocal_framework.platforms_handlers.handler_input import HandlerInput

    def __init__(self, parent_handler_input: HandlerInput):
        self.parent_handler_input = parent_handler_input
        self.request = Request()
        self.response = Response()

    def is_launch_request(self) -> bool:
        return self.request.is_launch_request()

    def is_in_intent_names(self, intent_names_list) -> bool:
        if not isinstance(intent_names_list, list):
            if isinstance(intent_names_list, str):
                intent_names_list = [intent_names_list]
            else:
                raise Exception(f"The intent_names_list must be a list or a str in order to be converted to a list,"
                                f"but it was a {type(intent_names_list)} object : {intent_names_list}")

        return self.request.is_in_intent_names(intent_names_list=intent_names_list)

    def say(self, text_or_ssml: str) -> None:
        self.response.say(text_or_ssml=text_or_ssml)

    def reprompt(self, text_or_ssml: str) -> None:
        self.response.say_reprompt(text_or_ssml=text_or_ssml)





import boto3
from inoft_vocal_framework.cli.core.policies import ATTACH_POLICY, ASSUME_POLICY


class CoreClients:
    def __init__(self):
        self.attach_policy = ATTACH_POLICY
        self.assume_policy = ASSUME_POLICY

        self._boto_session = None

        self._lambda_client = None
        self._s3_client = None
        self._dynamodb_client = None
        self._api_gateway_client = None
        self._events_client = None

        east_config = boto3.session.Config(region_name="us-east-1")
        self.acm_client = boto3.client("acm", config=east_config)
        self.iam_client = boto3.client("iam")
        self.iam_resource = boto3.resource("iam")
        self.iam = self.boto_session.client("iam")
        self.cloudformation_client = boto3.client("cloudformation")
        self.cloudwatch = boto3.client("cloudwatch")

    @property
    def boto_session(self):
        if self._boto_session is None:
            self._boto_session = boto3.Session()
        return self._boto_session

    @property
    def lambda_client(self):
        if self._lambda_client is None:
            self._lambda_client = boto3.client("lambda")
        return self._lambda_client

    @property
    def s3_client(self):
        if self._s3_client is None:
            self._s3_client = boto3.client("s3")
        return self._s3_client

    @property
    def dynamodb_client(self):
        if self._dynamodb_client is None:
            self._dynamodb_client = boto3.client("dynamodb")
        return self._dynamodb_client

    @property
    def api_gateway_client(self):
        if self._api_gateway_client is None:
            self._api_gateway_client = boto3.client("apigatewayv2")
        return self._api_gateway_client

    @property
    def events_client(self):
        if self._events_client is None:
            self._events_client = boto3.client("events")
        return self._events_client



from setuptools import setup

setup(
    name='inoftvocal',
    version='0.61.4',
    packages=['inoft_vocal_framework', 'inoft_vocal_framework.cli', 'inoft_vocal_framework.cli.core',
              'inoft_vocal_framework.utils',
              'inoft_vocal_framework.speechs', 'inoft_vocal_framework.databases',
              'inoft_vocal_framework.databases.dynamodb', 'inoft_vocal_framework.skill_builder',
              'inoft_vocal_framework.platforms_handlers', 'inoft_vocal_framework.platforms_handlers.alexa_v1',
              'inoft_vocal_framework.platforms_handlers.alexa_v1.response',
              'inoft_vocal_framework.platforms_handlers.alexa_v1.audioplayer',
              'inoft_vocal_framework.platforms_handlers.simulator',
              'inoft_vocal_framework.platforms_handlers.simulator.request_samples',
              'inoft_vocal_framework.platforms_handlers.simulator.request_samples.alexa',
              'inoft_vocal_framework.platforms_handlers.simulator.request_samples.google',
              'inoft_vocal_framework.platforms_handlers.dialogflow_v1',
              'inoft_vocal_framework.platforms_handlers.samsungbixby_v1',
              'inoft_vocal_framework.platforms_handlers.siri',
              'inoft_vocal_framework.platforms_handlers.endpoints_providers'],
    install_requires=["PyYAML", "cerberus", "boto3", "click"],
    entry_points={
        'console_scripts': [
            'inoft = inoft_vocal_framework.cli.cli_index:cli',
            'inoftvocal = inoft_vocal_framework.cli.cli_index:cli',
        ],
    },
    url='https://github.com/Robinson04/inoft_vocal_framework',
    download_url='https://github.com/Robinson04/inoft_vocal_framework/archive/0.61.0.tar.gz',
    license='MIT',
    author='Inoft',
    author_email='robinson@inoft.com',
    description='Create advanced cross-platform skills for Alexa, Google Assistant and Samsung Bixby',
)




ASSUME_POLICY = """{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "",
      "Effect": "Allow",
      "Principal": {
        "Service": [
          "apigateway.amazonaws.com",
          "lambda.amazonaws.com",
          "events.amazonaws.com"
        ]
      },
      "Action": "sts:AssumeRole"
    }
  ]
}"""

ATTACH_POLICY = """{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Effect": "Allow",
            "Action": [
                "logs:*"
            ],
            "Resource": "arn:aws:logs:*:*:*"
        },
        {
            "Effect": "Allow",
            "Action": [
                "lambda:InvokeFunction"
            ],
            "Resource": [
                "*"
            ]
        },
        {
            "Effect": "Allow",
            "Action": [
                "xray:PutTraceSegments",
                "xray:PutTelemetryRecords"
            ],
            "Resource": [
                "*"
            ]
        },
        {
            "Effect": "Allow",
            "Action": [
                "ec2:AttachNetworkInterface",
                "ec2:CreateNetworkInterface",
                "ec2:DeleteNetworkInterface",
                "ec2:DescribeInstances",
                "ec2:DescribeNetworkInterfaces",
                "ec2:DetachNetworkInterface",
                "ec2:ModifyNetworkInterfaceAttribute",
                "ec2:ResetNetworkInterfaceAttribute"
            ],
            "Resource": "*"
        },
        {
            "Effect": "Allow",
            "Action": [
                "s3:*"
            ],
            "Resource": "arn:aws:s3:::*"
        },
        {
            "Effect": "Allow",
            "Action": [
                "kinesis:*"
            ],
            "Resource": "arn:aws:kinesis:*:*:*"
        },
        {
            "Effect": "Allow",
            "Action": [
                "sns:*"
            ],
            "Resource": "arn:aws:sns:*:*:*"
        },
        {
            "Effect": "Allow",
            "Action": [
                "sqs:*"
            ],
            "Resource": "arn:aws:sqs:*:*:*"
        },
        {
            "Effect": "Allow",
            "Action": [
                "dynamodb:*"
            ],
            "Resource": "arn:aws:dynamodb:*:*:*"
        },
        {
            "Effect": "Allow",
            "Action": [
                "route53:*"
            ],
            "Resource": "*"
        }
    ]
}"""